#! /usr/bin/env python

#A. Bayegan
#This program uses forward and backward partition functions computed in Overcode software to 
#compute codon usage index for non-overlapping as well as overlapping reading frames.

import os,sys
#from ../Overcode import *
from aminoAcidAndGeneticCodes import genCode,aa2codon,aaCode

DEBUG=0

def computeCodoneProbFromGenesOverlap(seqList,rf):
#compute probability of codon w in the input sequence list 'seqList' for the overlapping reading frames given in rf. For each reading frame of rf
#compute the freq(w)/freq(syn(w)) in the corresponding reading frame. 
#Output is dictionary p_gene where keys are codons and values are sum of the probabilities over all reading frames.
	n = len(seqList[0])
	#-------------------initialize the codon frequency dictionary
	freq = {}
	for cod in genCode.keys():
		freq[cod] = 0
	#------------------check the length of the sequences to be 3k+2-----------------------------------
	m = n%3
	l = [] 
	if m!=2:
		for seq in seqList:
			l.append(seq[:n-m-1])
		print "WARNING: the last %d nucleotides ignored in computing codon usage from genes!" % (m+1)
		seqList = l
		if l==[]:
			print 'error: sequences are to short!'
			sys.exit(1)
		n = len(seqList[0])
	#------------------calculate frequency of codons in all reading frames --------------------------------
	for i in rf:
		for seq in seqList:
			if i[0]=='+': #positive strands
				r=int(i)
				for idx in range(r,n-2+r,3): #seq[0:n-2], seq[1:n-1], seq[2,n] code in reading frames +0, +1 and +2 respectively
					codon = seq[idx:idx+3]
					freq[codon] += 1
			elif i[0]=='-': #negative strands
				r=abs(int(i))
				for idx in range((n-2+r),r,-3): #seq[0:n-2], seq[1:n-1], seq[2,n] code in reading frames +0, +1 and +2 respectively
					codon = seq[idx-3:idx][::-1]
					freq[codon] += 1
	if DEBUG: 
		for k in sorted(freq.keys()):
			if freq[k]!=0:
				print k,freq[k]
	#----------------calculate frequency of synonymous codons in all reading frames-------------------------------
	freq_syn={}
	for codon in genCode.keys(): 
		freq_syn[codon] = 0
	for codon in freq.keys():
		for syncod in aa2codon[aaCode[genCode[codon].upper()]]:
			freq_syn[codon] += freq[syncod]
	if DEBUG: 
		for k in sorted(freq_syn.keys()):
			if freq_syn[k]!=0:
				print k,aa2codon[aaCode[genCode[k].upper()]],freq[k],freq_syn[k]
	#---------------calculate the codon usage index-----------------------------------------------------------------
	p = {}
	for codon in freq.keys():
		if freq_syn[codon]!=0:
			p[codon] = freq[codon]/float(freq_syn[codon])
	print p
	return p

def computeExpectedCodonProb(rf,peptides,ZF,ZB,filename=None,threshold=1):
#compute probability of codon w in the set of all sequences that translate 'peptides' in the corresponding reading frames.

	n  = len(peptides['+0'])
	#----------Initialize freq ---------------
	freq = {}
	for codon in genCode.keys():
		freq[codon] = 0
	#-----------calculate frequency of codons in all reading frames of interest -------------------
	for k in range(n):
		hexa = [peptides['+0'][k], peptides['+1'][k], peptides['+2'][k], peptides['-0'][n-k-1],peptides['-1'][n-k-1],peptides['-2'][n-k-1]]
		L = createListOfCompatible5mers(hexa,filename,threshold)
		for s in L:
			if '+0' in rf : freq[s[0:3]]+=1
			if '+1' in rf : freq[s[1:4]]+=1
			if '+2' in rf : freq[s[2:5]] +=1
			if '-0' in rf : freq[s[0:3][::-1]]+=1
			if '-1' in rf : freq[s[1:4][::-1]]+=1
			if '-2' in rf : freq[s[2:5][::-1]] +=1
	#----------calculate frequency of synonymous codons-----------------------------------------
	freq_syn={}
	for codon in genCode.keys(): 
		freq_syn[codon] = 0
	for codon in freq.keys():
		for syncod in aa2codon[aaCode[genCode[codon].upper()]]:
			freq_syn[codon] += freq[syncod]
	if DEBUG: 
		for k in sorted(freq_syn.keys()):
			if freq_syn[k]!=0:
				print k,aa2codon[aaCode[genCode[k].upper()]],freq[k],freq_syn[k]
	 
	return


if __name__=='__main__':
		computeCodoneProbFromGenesOverlap(['AAAAA','AAGCC'],['+0','+1','+2','-0','-1','-2'])
		
		
		
		
