#! /usr/bin/env python
#generate pi and n matrices from the input multiple alignment to plug in into sabath program.
#pi is the codon frequency matrix and n is the frequency of transitions between codons which are used in the
#likelihood function to compute codon transition probability matrixe.

import os,sys,numpy

stopcodon = ['UGA','UAA','UAG']
DEBUG=1	

def seq2int(seq):
	intseq = numpy.zeros((1,len(seq)))
	for i in range(len(seq)):
		intseq[0][i] = nuctoint(seq[i])
	return intseq

def codon2int(codonseq):
	return nuctoint(codonseq[0])+(nuctoint(codonseq[1])*4) + (nuctoint(codonseq[2])*16)

def nuctoint(char):
	if(char=='A'): return 0
	if(char=='C'): return 1
	if(char=='G'): return 2
	if(char=='U' or char=='T' ): return 3

#parse fasta file and get the sequences in all reading frames
def parseFasta(fname):
	list1=[]
	list2=[]
	list3=[]
	f=0
	infile = open(fname,'r')
	line = infile.readline()
	sid = line.strip()[1:]
	line = infile.readline()
	seq=''
	while line:
		if line[0]=='>':
			list1.append(seq[:-1])
			if f==0:
				n=len(seq)
				f=1
			if(len(seq)!=n):
				print ("all of the sequences in the alignment must have the same length")
				sys.exit(1)
			list2.append(seq[1:])
			list3.append(seq[2:len(seq)-2])
			sid = line.strip()[1:]
			seq = ''
		else:
			seq += line.strip()
		line= infile.readline()
	#if(len(seq)%3!=0):
	#	print "Error: The input sequences should have complete codon length (length%3=0)"
	#	sys.exit(1)
	list1.append(seq)
	list2.append(seq[1:])
	list3.append(seq[2:])
	infile.close()
	return list1,list2,list3

def getFreq3x4(l):
	f = numpy.zeros((3,4))
	for seq in l:
		n = len(seq)
		for i in range(0,3):
			for j in range(i,n,3):
				if(seq[i]!='-'):
					f[i][nuctoint(seq[j])] +=  1
	for i in range(0,3):
		for j in range(0,4):
			f[i][j] = f[i][j]/(len(l)*(n/3))
	return f

def hexa_PiF3x4(l):
	infile = open('hexa_idx.mat','r')
	hexalist = infile.readlines()
	f = getFreq3x4(l)
	hexaPi = numpy.zeros(3529)
	for i in range(3529):
		hexa = hexalist[i]
		hexaPi[i] = f[1][int(hexa.split()[0])-1]*f[2][int(hexa.split()[1])-1]*f[0][int(hexa.split()[2])-1]*f[1][int(hexa.split()[3])-1]*f[2][int(hexa.split()[4])-1]*f[0][int(hexa.split()[5])-1]
		#print hexa
		#print i,hexaPi[i]
	return hexaPi


def getPi3x4(l):
	f = getFreq3x4(l)
	pi = numpy.zeros((1,64))
	m=0
	#print (f)
	for i in range(4):
		for j in range(4):
			for k in range(4):
				pi[0][m]= (f[0][k]) * (f[1][j]) * (f[2][i])
				m+=1
	pi = numpy.delete(pi,35,1)
	pi = numpy.delete(pi,11,1)
	pi = numpy.delete(pi,3,1)
	#print pi
	return pi	

def getnij_slow(seqlist):
	m = len(seqlist)
	nij = numpy.zeros((64,64))
	counter = 0 
	for i in range(m):
		for j in range(i+1,m):
			seq1 = seqlist[i]
			seq2 = seqlist[j]
			counter=counter+1
			for k in range(0,len(seq1),3):
				#print seq1,seq2
				#print seq1[k:k+3],seq2[k:k+3]
				if( (seq1[k:k+3] in stopcodon) or (seq2[k:k+3] in stopcodon)):
					print("Warning: Stop codon %s or %s at position %d of sequence %s or %s!" % (seq1[k:k+3],seq2[k:k+3],k,seq1,seq2))
				if('-' not in seq1[k:k+3] and '-' not in seq2[k:k+3]):
					seq1intcodon=codon2int(seq1[k:k+3])
					seq2intcodon=codon2int(seq2[k:k+3])				
					nij[seq1intcodon][seq2intcodon] += 1 
					nij[seq2intcodon][seq1intcodon] = nij[seq1intcodon][seq2intcodon] 
					#print seq1intcodon,seq2intcodon,seq1[k:k+3],seq2[k:k+3],nij[seq1intcodon][seq2intcodon]
#remove stop codons. The order of removing is important.
	nij = numpy.delete(nij,35,0)
	nij = numpy.delete(nij,35,1)
	nij = numpy.delete(nij,11,0)
	nij = numpy.delete(nij,11,1)
	nij = numpy.delete(nij,3,0)
	nij = numpy.delete(nij,3,1)
	#print counter 	
	for i in range(61):
		for j in range(61):
			nij[i][j] /= counter 
			#if nij[i][j]!=0:
				#print i,j,nij[i][j]
				
	return nij 



def getnij(seqlist):
	n = numpy.zeros((64,64)) # identical to sites[](sabath code) and n_ij(ML formulation)
	L = len(seqlist[0])
	M = len(seqlist)
	codCount = getCodonCount(seqlist)
	for i in range(64):
		for j in range(64):
			for k in range(L/3):
				if(codCount[i][k]!=0 and codCount[j][k]!=0 ):
					if(i!=j):
						n[i][j] += (codCount[i][k]) * (codCount[j][k]) 
					else:
						n[i][j] += (codCount[i][k]-1) * (codCount[j][k])/2
					#print i,j,k,codCount[i][k] * codCount[j][k]
			n[i][j] /= (M*(M-1))/2
			
	n = numpy.delete(n,35,0)
	n = numpy.delete(n,35,1)
	n = numpy.delete(n,11,0)
	n = numpy.delete(n,11,1)
	n = numpy.delete(n,3,0)
	n = numpy.delete(n,3,1)
	return n

def getCodonCount(seqlist):
	n=len(seqlist[0])
	codCount = numpy.zeros((64,n/3))
	for seq in seqlist:
		for k in range(0,n,3):
			if( (seq[k:k+3] in stopcodon)):
				print("Warning: Stop codon %s or %s at position %d of sequence %s or %s!" % (seq1[k:k+3],seq2[k:k+3],k,seq1,seq2))
			if('-' not in seq[k:k+3]):
				#print seq[k:k+3],codon2int(seq[k:k+3]),k
				codCount[codon2int(seq[k:k+3])][k/3] += 1
	#if(DEBUG==1): print codCount
	return codCount

def printNonZeroArray(arr,n,m):
	for i in range(n):
		for j in range(m):
			if(arr[i][j]!=0):print i,j,arr[i][j]
	return

def matGenerator(fname,outname):
	orf1,orf2,orf3 = parseFasta(fname)
	i='A'
	#print (len(orf1),orf1[0],len(orf2),orf2[0])
	for l in [orf1,orf2]:
		pi = getPi3x4(l)
		numpy.savetxt(outname+'_pi'+i+'.txt',pi,fmt='%.10f')
		nij = getnij(l)
		numpy.savetxt(outname+'_n'+i+'.txt',nij,fmt='%.10f')		
		hexa = hexa_PiF3x4(l)
		numpy.savetxt(outname+'_hexpi'+i+'.txt',hexa,fmt='%.10f')
		i='B'
	#~ if(DEBUG==1):
		#~ n1 = getnij_slow(orf1)
		#~ n2 = getnij(orf1)
		#~ if numpy.array_equal(n1,n2):
			#~ print 'n1 and n2 are equal'
		#~ print 'n1'
		#~ printNonZeroArray(n1,61,61)
		#~ print 'n2'
		#~ printNonZeroArray(n2,61,61)
