function [w1 w2 tp k like] = dndsml_phase0_opp(seq1,seq2,a1,a2,b1,b2)

global indQ sites1 codon_pi1;

[indQ stop_codons] = build_indQ_opp_phase0();

sites1 = get_sites(seq1, seq2);

sites1(stop_codons,:) = [];
sites1(:,stop_codons) = [];

codon_pi1 = f3x4(a1, a2);

codon_pi1(stop_codons) = [];

codon_pi1 = repmat(codon_pi1,length(codon_pi1),1);

options = optimset('LargeScale','off','Display','off');
params = fmincon(@(x)get_likelihood_ov_phase0(x(1),x(2),x(3),x(4)),...
    [1 1 1 1],[],[],[],[],[0.001 0.001 0.001 0.001],[50 50 50 5],[],options);

k = params(1);
w1 = params(2);
w2 = params(3);
tp = params(4);

like = get_likelihood_ov_phase0(k,w1,w2,tp);



function [indQ stop_codons]  = build_indQ_opp_phase0()
% returns the indices for the matrix Q for phase 0 opposite-strand overlap 
% 
% the possible states are as follows:
%   stop codon in either genes: 0
%   synonymous in gene 1, synonymous in gene 2, transversions: 1
%   synonymous in gene 1, synonymous in gene 2, transitions: 2
%   nonsynonymous in gene 1, synonymous in gene 2, transversions: 3
%   nonsynonymous in gene 1, synonymous in gene 2, transitions: 4
%   synonymous in gene 1, nonsynonymous in gene 2, transversions: 5
%   synonymous in gene 1, nonsynonymous in gene 2, transitions: 6
%   nonsynonymous in gene 1, nonsynonymous in gene 2, transversions: 7
%   nonsynonymous in gene 1, nonsynonymous in gene 2, transitions: 8

codons = [1:64];
aa = intc2intaa(codons);
intcodon = reshape(intc2intnt(codons),3,length(codons))';

for i=1:64
    rc_intcodon(i,:) = revcom(intcodon(i,:));
    rc_aa(i) = intnt2intaa(revcom(intcodon(i,:)));
end

stop_codons = [find(rc_aa==24) find(aa==24)];

indQ = zeros(length(intcodon),length(intcodon));

for i = 1:length(intcodon)
    diffs = [intcodon(i,1)~=intcodon(:,1) ...
             intcodon(i,2)~=intcodon(:,2) ...
             intcodon(i,3)~=intcodon(:,3)];
    trans = [rem(intcodon(i,1),2)~=rem(intcodon(:,1),2) ...
             rem(intcodon(i,2),2)~=rem(intcodon(:,2),2) ...
             rem(intcodon(i,3),2)~=rem(intcodon(:,3),2)];
    ind = sum(diffs,2) == 1;
    syn1 = aa(i) == aa;
    syn2 = rc_aa(i) == rc_aa;
    indQ(i,any(trans&diffs,2)&ind&syn1'&syn2') = 1;
    indQ(i,~any(trans&diffs,2)&ind&syn1'&syn2') = 2;
    indQ(i,any(trans&diffs,2)&ind&~syn1'&syn2') = 3;
    indQ(i,~any(trans&diffs,2)&ind&~syn1'&syn2') = 4;
    indQ(i,any(trans&diffs,2)&ind&syn1'&~syn2') = 5;
    indQ(i,~any(trans&diffs,2)&ind&syn1'&~syn2') = 6;
    indQ(i,any(trans&diffs,2)&ind&~syn1'&~syn2') = 7;
    indQ(i,~any(trans&diffs,2)&ind&~syn1'&~syn2') = 8;
end

indQ(stop_codons,:) = [];
indQ(:,stop_codons) = [];



function sites = get_sites(seq1, seq2)
% returns a matrix with site counts for each codon pair
% skips gaps
if length(seq1)~= length(seq2)
    'Warning: unequal sequence lengths'
end
sites = zeros(64,64);
int_c1 = intnt2intc(seq1);
int_c2 = intnt2intc(seq2);
len = min(length(int_c1),length(int_c2));
for i=1:len
    sites(int_c1(i),int_c2(i)) = sites(int_c1(i),int_c2(i)) + 1;
end

% % remove stop codons entries
% stop_codons = intnt2intc([4 1 1 4 1 3 4 3 1]);
% sites(stop_codons,:) = [];
% sites(:,stop_codons) = [];




function [K_HKY85, t] = HKY85_F84(s1,s2,seq_length)
% Hasegawa distance taken from seqpdist modified according to Yang Nielsen
%
% Note: nuleotide frequencies are averaged between s1 and s2
%
% Let P and Q be the transition and transversional difference proportions
% and g the nucleotide frequencies over all the sequences:
%
% gr = g(1)+g(3);  % purines frequency
% gy = g(2)+g(4);  % pyrimidines frequency
% pr = g(1)*g(3);  % purines product
% py = g(2)*g(4);  % pyrimidines product
%
% Reference: Tamura and  Nei (1993) Molecular Biology and Evolution
% Reference: Yang and Nielsen(2000) Molecular Biology and Evolution

h = (s1(1:seq_length) ~='-') & (s2(1:seq_length)~='-');
map('ACGT- ') = 1:6;
temp1 = upper(char(s1(1:seq_length)));
temp2 = upper(char(s2(1:seq_length)));
g1 = accumarray(map(temp1(:))',1);
g2 = accumarray(map(temp2(:))',1);
g = (g1(1:4)+g2(1:4))/2;
g = g/ sum(g);
X  = accumarray(double([mynt2int(s1(h))',mynt2int(s2(h))']),1,[4 4]);
numPairs = sum(X(:));
numAGTransitions = sum(X([3 9]));
numCTTransitions = sum(X([8 14]));
numTransitions = numCTTransitions + numAGTransitions;
numTransversions = numPairs - sum(diag(X)) - numTransitions;
P = numTransitions / numPairs;
Q = numTransversions / numPairs;
gr = g(1)+g(3); % purines frequency
gy = g(2)+g(4); % pyrimidines frequency
pr = g(1)*g(3);
py = g(2)*g(4);
a = -log(max(eps,((2*(py+pr)) + ...
    ((2*(((py*gr)/gy)+((pr*gy)/gr))) * (1-(Q/(2*gy*gr)))) - P) ...
    / (2*((py/gy) + (pr/gr)))));
b = -log(max(eps,1-(Q/(2*gy*gr))));
K_F84 = a/b-1;
K_HKY85 = 1 + (((py/gy + pr/gr)*K_F84) / (py + pr));
t = (4*py*(1+K_F84/gy) + 4*pr*(1+K_F84/gr) + 4*gy*gr) * b;




function f = get_codon_frequencies(seq)

f = zeros(1,64);
c = intnt2intc(seq);

for i=1:64
    f(i) = length(find(c==i));
end

function freq = freq3x4(seq)
% calculate the F3x4 model frequencies

codons = reshape(seq,3,length(seq)/3);
freq = zeros(3,4);
for i=1:3
    for j=1:4
        freq(i,j) = sum(codons(i,:)==j)/length(codons(i,:));
    end
end


function f = f3x4(seq1,seq2)
% calculate the average frequenct of codons in the two sequences according to F3x4 model
f = zeros(1,64);

freq1 = freq3x4(seq1);
freq2 = freq3x4(seq2);

for i=1:64
    cod = intc2intnt(i);
    f(i) = mean([freq1(1,cod(1))*freq1(2,cod(2))*freq1(3,cod(3)),...
                freq2(1,cod(1))*freq2(2,cod(2))*freq2(3,cod(3))]);
end



function like = get_likelihood_ov_phase0(kappa,omega1,omega2,t)

global indQ sites1 codon_pi1;

Q1 = indQ;

sstv = (Q1 == 1);
ssts = (Q1 == 2);
nstv = (Q1 == 3);
nsts = (Q1 == 4);
sntv = (Q1 == 5);
snts = (Q1 == 6);
nntv = (Q1 == 7);
nnts = (Q1 == 8);

Q1(sstv) = codon_pi1(sstv);
Q1(ssts) = kappa*codon_pi1(ssts);
Q1(nstv) = omega1*codon_pi1(nstv);
Q1(nsts) = omega1*kappa*codon_pi1(nsts);
Q1(sntv) = omega2*codon_pi1(sntv);
Q1(snts) = omega2*kappa*codon_pi1(snts);
Q1(nntv) = omega1*omega2*codon_pi1(nntv);
Q1(nnts) = omega1*omega2*kappa*codon_pi1(nnts);

Q1 = Q1 + diag(-sum(Q1,2));
Q1 = Q1 ./ -sum(diag(codon_pi1).*diag(Q1));

P1 = expm(Q1 * t);

like = -sum(sum(sites1 .* log(P1.*codon_pi1)));% - sum(sum(sites2 .* log(P1*codon_pi1)));


