function opp_phase2_likelihood_variables()

stop_codons = intnt2intc([4 1 1 4 1 3 4 3 1]);
hexas = get_hexas();

[indQ hexas_with_stop] = build_indQ_ov_phase2_opp(hexas);

% remove stop codons entries
indQ(hexas_with_stop,:) = [];
indQ(:,hexas_with_stop) = [];

hexas(hexas_with_stop,:) = [];

codon_index = intnt2intc(reshape(hexas(:,2:4)',1,length(hexas)*3));
avg_mat = zeros(64,length(hexas));

for i=[1:64]
    ii = find(codon_index==i);
    avg_mat(i,ii) = 1;
end

avg_mat(stop_codons,:) = [];


save('opp_phase2_likelihood_variables','indQ','avg_mat','hexas')


function [indQ hexas_with_stop] = build_indQ_ov_phase2_opp(hexas)
% returns the indices for the matrix Q for phase 1 overlap where
% the codons of gene 1 are in positions 2-4 in the hexa
% the possible states are as follows:
%   stop codon in either genes: 0
%   synonymous in gene 1, synonymous in gene 2, traversions: 1
%   synonymous in gene 1, synonymous in gene 2, transitions: 2
%   nonsynonymous in gene 1, synonymous in gene 2, traversions: 3
%   nonsynonymous in gene 1, synonymous in gene 2, transitions: 4
%   synonymous in gene 1, nonsynonymous in gene 2, traversions: 5
%   synonymous in gene 1, nonsynonymous in gene 2, transitions: 6
%   nonsynonymous in gene 1, nonsynonymous in gene 2, traversions: 7
%   nonsynonymous in gene 1, nonsynonymous in gene 2, transitions: 8


codons1 = zeros(4^6,3);
codons2 = zeros(4^6,3);
codons3 = zeros(4^6,3);
aa1 = zeros(4^6,1);
aa2 = zeros(4^6,1);
aa3 = zeros(4^6,1);

hexas_with_stop = [];

for i=1:length(hexas)
    codons1(i,:) = revcom(hexas(i,1:3));
    codons2(i,:) = hexas(i,2:4);
    codons3(i,:) = revcom(hexas(i,4:6));
    aa1(i) = intnt2intaa(codons1(i,:));
    aa2(i) = intnt2intaa(codons2(i,:));
    aa3(i) = intnt2intaa(codons3(i,:));
    if aa1(i) == 24 || aa2(i) == 24 || aa3(i) == 24
        hexas_with_stop = [hexas_with_stop i];
    end
end
aa = [aa1, aa2, aa3];
indQ = sparse(length(hexas),length(hexas));

for i = 1:length(hexas)
    diffs = [hexas(i,1)~=hexas(:,1) ...
             hexas(i,2)~=hexas(:,2) ...
             hexas(i,3)~=hexas(:,3) ...
             hexas(i,4)~=hexas(:,4) ...
             hexas(i,5)~=hexas(:,5) ...
             hexas(i,6)~=hexas(:,6)];
    trans = [rem(hexas(i,3),2)~=rem(hexas(:,3),2) ...
             rem(hexas(i,4),2)~=rem(hexas(:,4),2) ...
             rem(hexas(i,5),2)~=rem(hexas(:,5),2)];
    ind = (sum(diffs,2) == 1)&(sum(diffs(:,2:4),2) == 1);
    syn1 = aa2(i) == aa2;
    syn2 = (aa1(i) == aa1)&(aa3(i) == aa3);
    indQ(i,any(trans&diffs(:,2:4),2)&ind&syn1&syn2) = 1;
    indQ(i,~any(trans&diffs(:,2:4),2)&ind&syn1&syn2) = 2;
    indQ(i,any(trans&diffs(:,2:4),2)&ind&~syn1&syn2) = 3;
    indQ(i,~any(trans&diffs(:,2:4),2)&ind&~syn1&syn2) = 4;
    indQ(i,any(trans&diffs(:,2:4),2)&ind&syn1&~syn2) = 5;
    indQ(i,~any(trans&diffs(:,2:4),2)&ind&syn1&~syn2) = 6;
    indQ(i,any(trans&diffs(:,2:4),2)&ind&~syn1&~syn2) = 7;
    indQ(i,~any(trans&diffs(:,2:4),2)&ind&~syn1&~syn2) = 8;
end

indQ(hexas_with_stop,:) = 0;
indQ(:,hexas_with_stop) = 0;



function hexas = get_hexas()

hexas = zeros(4^6,6);
for i=0:3
    for j=0:3
        for k=0:3
            for l=0:3
                for m=0:3
                    for n=0:3
                        hex = 1+[i,j,k,l,m,n]*[1,4,16,64,256,1024]';
                        hexas(hex,:) = [i j k l m n]+1;
                    end
                end
            end
        end
    end
end   
