function ss_phase2_likelihood_variables()
% exactly the same as phase 1 but with opposite phase
stop_codons = intnt2intc([4 1 1 4 1 3 4 3 1]);
hexas1 = get_hexas();

[indQ1 hexas_with_stop1] = build_indQ_ov_phase2(hexas1);

% remove stop codons entries
indQ1(hexas_with_stop1,:) = [];
indQ1(:,hexas_with_stop1) = [];

hexas1(hexas_with_stop1,:) = [];

codon_index1 = intnt2intc(reshape(hexas1(:,2:4)',1,length(hexas1)*3));
avg_mat1 = zeros(64,length(hexas1));

for i=[1:64]
    ii = find(codon_index1==i);
    avg_mat1(i,ii) = 1;
end
avg_mat1(stop_codons,:) = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%
hexas2 = get_hexas();

[indQ2 hexas_with_stop2] = build_indQ_ov_phase1(hexas2);

% remove stop codons entries
indQ2(hexas_with_stop2,:) = [];
indQ2(:,hexas_with_stop2) = [];

hexas2(hexas_with_stop2,:) = [];

codon_index2 = intnt2intc(reshape(hexas2(:,3:5)',1,length(hexas2)*3));
avg_mat2 = zeros(64,length(hexas2));

for i=[1:64]
    ii = find(codon_index2==i);
    avg_mat2(i,ii) = 1;
end
avg_mat2(stop_codons,:) = [];

w = ones(1,64);
w([4,12,36]) = 0;

rand_seq = randCodonDNAseq(10000,w);


save('ss_phase2_likelihood_variables','indQ1','indQ2','avg_mat1','avg_mat2','hexas1','hexas2')


function freq = freq3x4(seq)
% calculate the F3x4 model frequencies

codons = reshape(seq,3,length(seq)/3);
freq = zeros(3,4);
for i=1:3
    for j=1:4
        freq(i,j) = sum(codons(i,:)==j)/length(codons(i,:));
    end
end

function f = f3x4(seq1,seq2)
% calculate the average frequenct of codons in the two sequences according to F3x4 model
f = zeros(1,64);

freq1 = freq3x4(seq1);
freq2 = freq3x4(seq2);

for i=1:64
    cod = intc2intnt(i);
    f(i) = mean([freq1(1,cod(1))*freq1(2,cod(2))*freq1(3,cod(3)),...
                freq2(1,cod(1))*freq2(2,cod(2))*freq2(3,cod(3))]);
end

stop_codons = intnt2intc([4 1 1 4 1 3 4 3 1]);
f(stop_codons) = [];



function f = hexa1_f3x4(seq,hexas)
% calculate the frequency of phase 1 hexas according to F3x4 model
% codons of gene 1 are in positions 3-5 in the hexa
f = zeros(1,length(hexas));

freq1 = freq3x4(seq);

for i=1:length(f)
    h = hexas(i,:);
    f(i) = freq1(2,h(1))*freq1(3,h(2))*freq1(1,h(3))*freq1(2,h(4))*freq1(3,h(5))*freq1(1,h(6));               
end



function [indQ hexas_with_stop] = build_indQ_ov_phase1(hexas)
% returns the indices for the matrix Q for phase 1 overlap where
% the codons of gene 1 are in positions 3-5 in the hexa
% the possible states are as follows:
%   stop codon in either genes: 0
%   synonymous in gene 1, synonymous in gene 2, traversions: 1
%   synonymous in gene 1, synonymous in gene 2, transitions: 2
%   nonsynonymous in gene 1, synonymous in gene 2, traversions: 3
%   nonsynonymous in gene 1, synonymous in gene 2, transitions: 4
%   synonymous in gene 1, nonsynonymous in gene 2, traversions: 5
%   synonymous in gene 1, nonsynonymous in gene 2, transitions: 6
%   nonsynonymous in gene 1, nonsynonymous in gene 2, traversions: 7
%   nonsynonymous in gene 1, nonsynonymous in gene 2, transitions: 8

hexas = get_hexas();

codons1 = zeros(4^6,3);
codons2 = zeros(4^6,3);
codons3 = zeros(4^6,3);
aa1 = zeros(4^6,1);
aa2 = zeros(4^6,1);
aa3 = zeros(4^6,1);

hexas_with_stop = [];

for i=1:length(hexas)
    codons1(i,:) = hexas(i,1:3);
    codons2(i,:) = hexas(i,3:5);
    codons3(i,:) = hexas(i,4:6);
    aa1(i) = intnt2intaa(codons1(i,:));
    aa2(i) = intnt2intaa(codons2(i,:));
    aa3(i) = intnt2intaa(codons3(i,:));
    if aa1(i) == 24 || aa2(i) == 24 || aa3(i) == 24
        hexas_with_stop = [hexas_with_stop i];
    end
end
aa = [aa1, aa2, aa3];
indQ = sparse(length(hexas),length(hexas));

for i = 1:length(hexas)
    diffs = [hexas(i,1)~=hexas(:,1) ...
             hexas(i,2)~=hexas(:,2) ...
             hexas(i,3)~=hexas(:,3) ...
             hexas(i,4)~=hexas(:,4) ...
             hexas(i,5)~=hexas(:,5) ...
             hexas(i,6)~=hexas(:,6)];
    trans = [rem(hexas(i,3),2)~=rem(hexas(:,3),2) ...
             rem(hexas(i,4),2)~=rem(hexas(:,4),2) ...
             rem(hexas(i,5),2)~=rem(hexas(:,5),2)];
    ind = (sum(diffs,2) == 1)&(sum(diffs(:,3:5),2) == 1);
    syn1 = aa2(i) == aa2;
    syn2 = (aa1(i) == aa1)&(aa3(i) == aa3);
    indQ(i,any(trans&diffs(:,3:5),2)&ind&syn1&syn2) = 1;
    indQ(i,~any(trans&diffs(:,3:5),2)&ind&syn1&syn2) = 2;
    indQ(i,any(trans&diffs(:,3:5),2)&ind&~syn1&syn2) = 3;
    indQ(i,~any(trans&diffs(:,3:5),2)&ind&~syn1&syn2) = 4;
    indQ(i,any(trans&diffs(:,3:5),2)&ind&syn1&~syn2) = 5;
    indQ(i,~any(trans&diffs(:,3:5),2)&ind&syn1&~syn2) = 6;
    indQ(i,any(trans&diffs(:,3:5),2)&ind&~syn1&~syn2) = 7;
    indQ(i,~any(trans&diffs(:,3:5),2)&ind&~syn1&~syn2) = 8;
end

indQ(hexas_with_stop,:) = 0;
indQ(:,hexas_with_stop) = 0;


function [indQ hexas_with_stop] = build_indQ_ov_phase2(hexas)
% returns the indices for the matrix Q for phase 2 overlap where
% the codons of gene 1 are position 2-4 in the hexa
% the possible states are as follows:
%   stop codon in either genes: 0
%   synonymous in gene 1, synonymous in gene 2, traversions: 1
%   synonymous in gene 1, synonymous in gene 2, transitions: 2
%   nonsynonymous in gene 1, synonymous in gene 2, traversions: 3
%   nonsynonymous in gene 1, synonymous in gene 2, transitions: 4
%   synonymous in gene 1, nonsynonymous in gene 2, traversions: 5
%   synonymous in gene 1, nonsynonymous in gene 2, transitions: 6
%   nonsynonymous in gene 1, nonsynonymous in gene 2, traversions: 7
%   nonsynonymous in gene 1, nonsynonymous in gene 2, transitions: 8

hexas = get_hexas();

codons1 = zeros(4^6,3);
codons2 = zeros(4^6,3);
codons3 = zeros(4^6,3);
aa1 = zeros(4^6,1);
aa2 = zeros(4^6,1);
aa3 = zeros(4^6,1);

hexas_with_stop = [];

for i=1:length(hexas)
    codons1(i,:) = hexas(i,1:3);
    codons2(i,:) = hexas(i,2:4);
    codons3(i,:) = hexas(i,4:6);
    aa1(i) = intnt2intaa(codons1(i,:));
    aa2(i) = intnt2intaa(codons2(i,:));
    aa3(i) = intnt2intaa(codons3(i,:));
    if aa1(i) == 24 || aa2(i) == 24 || aa3(i) == 24
        hexas_with_stop = [hexas_with_stop i];
    end
end
aa = [aa1, aa2, aa3];
indQ = sparse(length(hexas),length(hexas));

for i = 1:length(hexas)
    diffs = [hexas(i,1)~=hexas(:,1) ...
             hexas(i,2)~=hexas(:,2) ...
             hexas(i,3)~=hexas(:,3) ...
             hexas(i,4)~=hexas(:,4) ...
             hexas(i,5)~=hexas(:,5) ...
             hexas(i,6)~=hexas(:,6)];
    trans = [rem(hexas(i,2),2)~=rem(hexas(:,2),2) ...
             rem(hexas(i,3),2)~=rem(hexas(:,3),2) ...
             rem(hexas(i,4),2)~=rem(hexas(:,4),2)];
    ind = (sum(diffs,2) == 1)&(sum(diffs(:,2:4),2) == 1);
    syn1 = aa2(i) == aa2;
    syn2 = (aa1(i) == aa1)&(aa3(i) == aa3);
    indQ(i,any(trans&diffs(:,2:4),2)&ind&syn1&syn2) = 1;
    indQ(i,~any(trans&diffs(:,2:4),2)&ind&syn1&syn2) = 2;
    indQ(i,any(trans&diffs(:,2:4),2)&ind&~syn1&syn2) = 3;
    indQ(i,~any(trans&diffs(:,2:4),2)&ind&~syn1&syn2) = 4;
    indQ(i,any(trans&diffs(:,2:4),2)&ind&syn1&~syn2) = 5;
    indQ(i,~any(trans&diffs(:,2:4),2)&ind&syn1&~syn2) = 6;
    indQ(i,any(trans&diffs(:,2:4),2)&ind&~syn1&~syn2) = 7;
    indQ(i,~any(trans&diffs(:,2:4),2)&ind&~syn1&~syn2) = 8;
end

indQ(hexas_with_stop,:) = 0;
indQ(:,hexas_with_stop) = 0;



function hexas = get_hexas()

hexas = zeros(4^6,6);
for i=0:3
    for j=0:3
        for k=0:3
            for l=0:3
                for m=0:3
                    for n=0:3
                        hex = 1+[i,j,k,l,m,n]*[1,4,16,64,256,1024]';
                        hexas(hex,:) = [i j k l m n]+1;
                    end
                end
            end
        end
    end
end   
